import { useSelector } from "react-redux";
import actionCreatorFactory from "typescript-fsa";
import { combineEpics } from "redux-observable";
import { reducerWithInitialState } from "typescript-fsa-reducers";
import { ignoreElements, map, tap } from "rxjs/operators";
import { ofAction } from "@datalayer/typescript-fsa-redux-observable";
import { cmdIds } from "./NotebookCommands";
export const notebookInitialState = {
    notebooks: new Map(),
};
/* Selectors */
export const selectNotebook = (uid) => useSelector((state) => {
    if (state.notebook) {
        return state.notebook.notebooks.get(uid);
    }
    return undefined;
});
export const selectNotebookModel = (uid) => useSelector((state) => {
    var _a;
    if (state.notebook) {
        return (_a = state.notebook.notebooks.get(uid)) === null || _a === void 0 ? void 0 : _a.model;
    }
    return undefined;
});
export const selectActiveCell = (uid) => useSelector((state) => {
    var _a;
    if (state.notebook) {
        return (_a = state.notebook.notebooks.get(uid)) === null || _a === void 0 ? void 0 : _a.activeCell;
    }
    return undefined;
});
export const selectNotebookPortals = (uid) => useSelector((state) => {
    var _a;
    if (state.notebook) {
        return (_a = state.notebook.notebooks.get(uid)) === null || _a === void 0 ? void 0 : _a.portals;
    }
    return undefined;
});
export const selectSaveRequest = (uid) => useSelector((state) => {
    var _a;
    if (state.notebook) {
        return (_a = state.notebook.notebooks.get(uid)) === null || _a === void 0 ? void 0 : _a.saveRequest;
    }
    return undefined;
});
export const selectNotebookPortalDisplay = (uid) => useSelector((state) => {
    var _a;
    if (state.notebook) {
        return (_a = state.notebook.notebooks.get(uid)) === null || _a === void 0 ? void 0 : _a.portalDisplay;
    }
    return undefined;
});
/* Actions */
export var ActionType;
(function (ActionType) {
    ActionType["ACTIVE_CELL_CHANGE"] = "notebook/ACTIVE_CELL_CHANGE";
    ActionType["ADD_PORTALS"] = "notebook/ADD_PORTALS";
    ActionType["CHANGE_CELL_TYPE"] = "notebook/CHANGE_CELL_TYPE";
    ActionType["CHANGE_KERNEL"] = "notebook/CHANGE_KERNEL";
    ActionType["DELETE"] = "notebook/DELETE";
    ActionType["DISPOSE"] = "notebook/DISPOSE";
    ActionType["INSERT_ABOVE"] = "notebook/INSERT_ABOVE";
    ActionType["INSERT_BELOW"] = "notebook/INSERT_BELOW";
    ActionType["INTERRUPT"] = "notebook/INTERRUPT";
    ActionType["KERNEL_STATUS_CHANGE"] = "notebook/KERNEL_STATUS_CHANGE";
    ActionType["MODEL_CHANGE"] = "notebook/MODEL_CHANGE";
    ActionType["NOTEBOOK_CHANGE"] = "notebook/NOTEBOOK_CHANGE";
    ActionType["RESET"] = "notebook/RESET";
    ActionType["RUN"] = "notebook/RUN";
    ActionType["RUN_ALL"] = "notebook/RUN_ALL";
    ActionType["SAVE"] = "notebook/SAVE";
    ActionType["SET_PORTALS"] = "notebook/SET_PORTALS";
    ActionType["SET_PORTAL_DISPLAY"] = "notebook/SET_PORTAL_DISPLAY";
    ActionType["UPDATE"] = "notebook/UPDATE";
})(ActionType || (ActionType = {}));
const actionCreator = actionCreatorFactory('jupyterNotebook');
export const notebookActions = {
    reset: actionCreator(ActionType.RESET),
    update: actionCreator(ActionType.UPDATE),
    notebookChange: actionCreator(ActionType.NOTEBOOK_CHANGE),
    modelChange: actionCreator(ActionType.MODEL_CHANGE),
    changeKernel: actionCreator(ActionType.CHANGE_KERNEL),
    activeCellChange: actionCreator(ActionType.ACTIVE_CELL_CHANGE),
    kernelStatusChanged: actionCreator(ActionType.KERNEL_STATUS_CHANGE),
    addPortals: actionCreator(ActionType.ADD_PORTALS),
    setPortals: actionCreator(ActionType.SET_PORTALS),
    setPortalDisplay: actionCreator(ActionType.SET_PORTAL_DISPLAY),
    dispose: actionCreator(ActionType.DISPOSE),
    save: actionCreator.async(ActionType.SAVE),
    run: actionCreator.async(ActionType.RUN),
    runAll: actionCreator.async(ActionType.RUN_ALL),
    interrupt: actionCreator.async(ActionType.INTERRUPT),
    insertAbove: actionCreator.async(ActionType.INSERT_ABOVE),
    insertBelow: actionCreator.async(ActionType.INSERT_BELOW),
    delete: actionCreator.async(ActionType.DELETE),
    changeCellType: actionCreator.async(ActionType.CHANGE_CELL_TYPE)
};
/* Epics */
const runEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.run.started), tap(action => {
    var _a, _b;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.commands.execute(cmdIds.run);
}), ignoreElements());
const runAllEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.runAll.started), tap(action => {
    var _a, _b;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.commands.execute(cmdIds.runAll);
}), ignoreElements());
const interruptEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.interrupt.started), tap(action => {
    var _a, _b;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.commands.execute(cmdIds.interrupt);
}), ignoreElements());
const insertAboveEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.insertAbove.started), tap(action => {
    var _a, _b, _c, _d;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.setDefaultCellType(action.payload.cellType);
    (_d = (_c = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _c === void 0 ? void 0 : _c.adapter) === null || _d === void 0 ? void 0 : _d.commands.execute(cmdIds.insertAbove);
}), ignoreElements());
const insertBelowEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.insertBelow.started), tap(action => {
    var _a, _b, _c, _d;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.setDefaultCellType(action.payload.cellType);
    (_d = (_c = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _c === void 0 ? void 0 : _c.adapter) === null || _d === void 0 ? void 0 : _d.commands.execute(cmdIds.insertBelow);
}), ignoreElements());
const deleteEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.delete.started), tap(action => {
    var _a, _b;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.commands.execute(cmdIds.deleteCells);
}), ignoreElements());
const changeCellTypeEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.changeCellType.started), tap(action => {
    var _a, _b;
    //      state$.value.notebook?.adapter?.commands.execute(cmdIds.toCode);
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.changeCellType(action.payload.cellType);
    /*
    NotebookActions.changeCellType(
      state$.value.notebook.notebooks.get(action.payload)?.adapter?.notebookPanel?.content!,
      action.payload
    );
    */
}), ignoreElements());
const saveEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.save.started), map(action => {
    var _a, _b;
    (_b = (_a = state$.value.notebook.notebooks.get(action.payload.uid)) === null || _a === void 0 ? void 0 : _a.adapter) === null || _b === void 0 ? void 0 : _b.commands.execute(cmdIds.save);
    return notebookActions.save.done({
        params: action.payload,
        result: action.payload,
    });
}));
export const notebookEpics = combineEpics(runEpic, runAllEpic, interruptEpic, insertAboveEpic, insertBelowEpic, deleteEpic, changeCellTypeEpic, saveEpic);
/* Reducers */
export const notebookReducer = reducerWithInitialState(notebookInitialState)
    .case(notebookActions.reset, (state, _) => {
    return notebookInitialState;
})
    .case(notebookActions.update, (state, updateUid) => {
    const notebooks = state.notebooks;
    let n = notebooks.get(updateUid.uid);
    if (n) {
        n = Object.assign(Object.assign({}, n), updateUid.partialState);
    }
    else {
        notebooks.set(updateUid.uid, {
            adapter: updateUid.partialState.adapter,
            portals: [],
        });
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.activeCellChange, (state, cellModelUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(cellModelUid.uid);
    if (n) {
        n.activeCell = cellModelUid.cellModel;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.modelChange, (state, modelUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(modelUid.uid);
    if (n) {
        n.model = modelUid.notebookModel;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.notebookChange, (state, notebookChangeUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(notebookChangeUid.uid);
    if (n) {
        n.notebookChange = notebookChangeUid.notebookChange;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.kernelStatusChanged, (state, kernelStatusUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(kernelStatusUid.uid);
    if (n) {
        n.kernelStatus = kernelStatusUid.kernelStatus;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.changeKernel, (state, kernelChange) => {
    var _a;
    const notebooks = state.notebooks;
    const n = notebooks.get(kernelChange.uid);
    if (n) {
        (_a = n.adapter) === null || _a === void 0 ? void 0 : _a.changeKernel(kernelChange.kernel);
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.addPortals, (state, portalsUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(portalsUid.uid);
    if (n) {
        n.portals = n.portals.concat(portalsUid.portals);
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.dispose, (state, uid) => {
    const notebooks = state.notebooks;
    notebooks.delete(uid);
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.setPortals, (state, portalsUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(portalsUid.uid);
    if (n) {
        n.portals = portalsUid.portals;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.setPortalDisplay, (state, portalDisplayUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(portalDisplayUid.uid);
    if (n) {
        n.portalDisplay = portalDisplayUid.portalDisplay;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.save.done, (state, dateUid) => {
    const notebooks = state.notebooks;
    const n = notebooks.get(dateUid.result.uid);
    if (n) {
        n.saveRequest = dateUid.result.date;
    }
    return Object.assign(Object.assign({}, state), { notebooks });
})
    .case(notebookActions.insertAbove.done, (state, _) => {
    return state;
})
    .case(notebookActions.insertBelow.done, (state, _) => {
    return state;
})
    .case(notebookActions.changeCellType.done, (state, _) => {
    return state;
})
    .case(notebookActions.run.done, (state, _) => {
    return state;
});
//# sourceMappingURL=NotebookState.js.map