var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { SessionManager } from '@jupyterlab/services';
import { UUID } from '@lumino/coreutils';
export class Kernel {
    constructor(options) {
        this._kernelManager = options.kernelManager;
        this._kernelName = options.kernelName;
        // Request the effective Jupyter Kernel.
        this._kernel = this.requestKernel();
    }
    requestKernel() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._kernelManager.ready;
            const sessionManager = new SessionManager({
                kernelManager: this._kernelManager,
                serverSettings: this._kernelManager.serverSettings,
                standby: 'never',
            });
            yield sessionManager.ready;
            const randomName = UUID.uuid4();
            this._session = yield sessionManager.startNew({
                path: randomName,
                name: randomName,
                type: this._kernelName,
            });
            yield this._session.kernel.info;
            this._id = this._session.kernel.id;
            return this._session.kernel;
        });
    }
    getJupyterKernel() {
        return this._kernel;
    }
    get id() {
        return this._id;
    }
    shutdown() {
        var _a;
        (_a = this._session.kernel) === null || _a === void 0 ? void 0 : _a.shutdown();
        //    this.getJupyterKernel().then(k => k.dispose());
    }
}
export default Kernel;
//# sourceMappingURL=Kernel.js.map